#include <avr/io.h>
#include <avr/interrupt.h>

#include <util/delay.h>

#include <stdio.h>
#include <string.h>

#include "i2cmaster.h"

#include "pff.h"

#define NUM_FILES 3

#include "sd.h"


FATFS fs; // sistemul de fisiere

#define BUF_FRONT	(buf[buf_front])
#define BUF_BACK	(buf[1 - buf_front])

uint16_t	buf[2][256];	// wave output buffers (double buffering)
const	 uint16_t	buf_size = 256;	// front and back buffer sizes
volatile uint16_t	buf_front = 0;	// front buffer index (current buffer used)
volatile uint16_t	buf_pos = 0;	// current buffer position
volatile uint16_t	buf_sync = 0;

void writeMessage(char* message, int len)
{
	FRESULT res;
	WORD bytes_written;
	res = pf_open("Loopsys/sound1.dat");
	
	res = pf_write(message, len, &bytes_written);
}

void writeBinary(uint16_t* buff, int len)
{
	FRESULT res;
	WORD bytes_written;
	
	res = pf_write(buff, len, &bytes_written);
}

void recordSample(uint16_t val)
{
	BUF_FRONT[buf_pos++] = val;
	
	if (buf_pos == buf_size) {
		while (i2c_write(val & 0xFF) != 0) {
		PORTD &= ~(1 << PD7);
		_delay_ms(1000);
		}
		PORTD |= (1 << PD7);
		while (i2c_write(val >> 8) != 0) {
			PORTD &= ~(1 << PD7);
			_delay_ms(1000);
		}
		writeBinary(BUF_FRONT, buf_size);
		buf_pos = 0;
	}
}

void IO_init(void)
{
	DDRD |= 1 << PD7;
	PORTD &= ~(1 << PD7);
}

void ADC_init(void)
{
	ADMUX    |= _BV(MUX0); 	
	ADMUX    |= _BV(MUX1);
	ADMUX    |= _BV(MUX2);
	ADCSRA   |= _BV(ADEN); // enable ADC
	ADCSRA   |= _BV(ADSC); // start work
	ADMUX    |= _BV(REFS0); // AREF = AVCC
	
	ADCSRA  |= _BV(ADPS0); // prescaler = 128
	ADCSRA  |= _BV(ADPS1);
	ADCSRA  |= _BV(ADPS2);
	ADCSRA  |= _BV(ADIE); // enable interrupt
	
	DDRA   &= ~(_BV(PA7)); // input
}

ISR(ADC_vect)
{
	PORTD &= ~(1 << PD7);
	uint16_t val = ADC;
	//recordSample(val);
	
	val += 32765 - 512;
	
	ADCSRA |= _BV(ADSC);
	
	PORTD |= (1 << PD7);
}

void timer0_start(void)
{
	// interrupt on compare A
	TIMSK0 |= (1 << OCIE0A);
	// CTC, top OCRA
	TCCR0B |= (0 << WGM02);
	TCCR0A |= (1 << WGM01) | (0 << WGM00);
	// prescaler 8
	TCCR0B |= (2 << CS00);
}

void timer0_stop(void)
{
	TCCR0B = 0;
	TCCR0A = 0;
	TIMSK0 = 0;
	OCR0A = 0;
	TCNT0 = 0;
}

BOOL continue_play()
{
	if((PINB & (1 << PB0)) == 0 && (PINB & (1 << PB1)) == 0)
		return FALSE;

	return TRUE;
}


ISR(TIMER2_COMPA_vect)
{
	;// maybe use later
}

void timer2_init(void)
{
	// set compare at each milisecond
	OCR2A = 124;
	// interrupt on compare A
	TIMSK2 |= (1 << OCIE2A);
	// CTC, top OCRA
	TCCR2B |= (0 << WGM22);
	TCCR2A |= (1 << WGM21) | (0 << WGM20);
	// prescaler 128
	TCCR2B |= (5 << CS20);
}

int parity = 0;
int nump = 20;

int main (void)
{
	IO_init();
	//ADC_init();
	
	//sei();
	
	i2c_init();
	
	
	i2c_start_wait(0x98); // 1001 1000 - address
	
	while (i2c_write(0x10) != 0) { // 0001 0000 - control byte
		;
	}
	
	/*
	unsigned int val;
	
	while (1) {
		val = 65000 / nump * parity++;
		parity = parity % nump;
			
		while (i2c_write(val & 0xFF) != 0) {
			PORTD &= ~(1 << PD7);
			_delay_ms(1000);
		}
		PORTD |= (1 << PD7);
		while (i2c_write(val >> 8) != 0) {
			PORTD &= ~(1 << PD7);
			_delay_ms(1000);
		}
		PORTD |= (1 << PD7);
	}
	*/
	FRESULT res;

	for(;;)
	{
		// mount filesystem
		
		res = pf_mount(&fs);
		
                
		if(res != FR_OK)
		{
			_delay_ms(500); // wait a while and retry
			
			continue;
		}
		
		
		res = pf_open("Loopsys/sound1.dat");
		
		PORTD |= (1 << PD7);
		_delay_ms(1000);
		
		ADCSRA |= _BV(ADSC);
		
		for(;;)
		{
			_delay_ms(200);
			break;
		}
		break;
	}

	return 0;
}
